@@ -3,8 +3,10 @@ package Data::Path;
 use 5.006001;
 use strict;
 use warnings;
+use Scalar::Util qw/reftype blessed/;
+use Carp;
 
-our $VERSION = '1.0';
+our $VERSION = '1.3.1';
 
 sub new {
 	my ($class,$data,$callback)=@_;
@@ -17,25 +19,30 @@ sub new {
 			{ key_does_not_exist            => $callback->{key_does_not_exist} ||
 				sub {
 					my ($data, $key, $index, $value, $rest )=@_;
-					die "key $key does not exists\n";
+					croak "key $key does not exists\n";
 				}
 
 			, index_does_not_exist         => $callback->{index_does_not_exist} ||
 				sub {
 					my ($data, $key, $index, $value, $rest )=@_;
-					die "key $key\[$index\] does not exists\n";
+					croak "key $key\[$index\] does not exists\n";
 				}
 
 			, retrieve_index_from_non_array => $callback->{retrieve_index_from_non_array} ||
 				sub {
 					my ($data, $key, $index, $value, $rest )=@_;
-					die "trie to retrieve an index $index from a no array value (in key $key)\n";
+					croak "trie to retrieve an index $index from a no array value (in key $key)\n";
 				}
 
 			, retrieve_key_from_non_hash    => $callback->{retrieve_key_from_non_hash} ||
 				sub {
 					my ($data, $key, $index, $value, $rest )=@_;
-					die "trie to retrieve a key from a no hash value (in key $key)\n";
+					croak "trie to retrieve a key from a no hash value (in key $key)\n";
+				}
+			, not_a_coderef_or_method    => $callback->{not_a_coderef_or_method} ||
+				sub {
+					my ($data, $key, $index, $value, $rest )=@_;
+					croak "tried to retrieve from a non-existant coderef or method: $key in $data";
 				}
 			}
 
@@ -51,7 +58,11 @@ sub get {
 
 	# get key till / or [
 	my $key = $1 if ( $rkey =~ s/^\/([^\/|\[]+)//o );
-	
+    croak 'malformed path expression'
+        unless $key;
+   
+    croak 'malformed array index request' 
+        if $rkey =~ /^\[([^\d]*)\]/;
 	# check index for index
 	my $index = $1 if ( $rkey =~ s/^\[(\d+)\]//o );
 
@@ -59,20 +70,29 @@ sub get {
 	my $rest  = $rkey;
 
 	# get key from data
-	my $value = $data->{$key};
-
-	# die if key does not exists and something after that is requested 
+	my $value;
+    if ($key =~ s/(\(\))$//) {
+        $self->{callback}->{not_a_coderef_or_method}->($data, $key, $index, $value, $rest)
+            unless exists $data->{$key} or (blessed $data && $data->can($key)); 
+        
+        $value = $data->{$key}->() if (exists $data->{$key});
+        $value = $data->$key() if blessed $data && $data->can($key);
+    } else {
+	   $value = $data->{$key};
+    }
+	
+    # croak if key does not exists and something after that is requested 
 	$self->{callback}->{key_does_not_exist}->($data, $key, $index, $value, $rest) 
 		if not exists $data->{$key} and $rest;
 
 	# check index 
 	if (defined $index) {
 
-		# die if index does not exists and something after that is requested 
+		# croak if index does not exists and something after that is requested 
 		$self->{callback}->{index_does_not_exist}->($data, $key, $index, $value, $rest) 
 			if not exists $value->[$index] and $rest;
 
-		if ( ref $value eq 'ARRAY' ) {
+		if ( reftype $value eq 'ARRAY' ) {
 			$value=$value->[$index];
 		} else {
 			$self->{callback}->{retrieve_index_from_non_array}->($data, $key, $index, $value, $rest);
@@ -81,7 +101,7 @@ sub get {
 
 	# check if last element is reached
 	if ($rest) {
-		if ( ref $value eq 'HASH' ) {
+		if ( reftype $value eq 'HASH' || blessed $value ) {
 			$value=$self->get($rest,$value);
 		} else {
 			$self->{callback}->{retrieve_key_from_non_hash}->($data, $key, $index, $value, $rest);
@@ -110,12 +130,15 @@ Data::Path - Perl extension for XPath like accessing from complex data structs
 			, { text => 'msg1' }
 			, { text => 'msg2' }
 			]
-	}
+	},
+    method => sub {'method text'}
   }
   
   my $hpath=Data::Path->new($hashdata);
   my $value= $hpath->get('/result/msg[1]/text');
-
+  my $value2 = $hpath->get('/method()');
+  
+  print "OK" if $value2 eq 'method text';
   print "OK" if $value eq 'msg1';
 
 
@@ -140,25 +163,30 @@ The default callbacks but you can overwrite this.
 	{ key_does_not_exist            =>
 		sub {
 			my ($data, $key, $index, $value, $rest )=@_;
-			die "key $key does not exists\n";
+			croak "key $key does not exists\n";
 		}
 
 	, index_does_not_exist         =>
 		sub {
 			my ($data, $key, $index, $value, $rest )=@_;
-			die "key $key\[$index\] does not exists\n";
+			croak "key $key\[$index\] does not exists\n";
 		}
 
 	, retrieve_index_from_non_array =>
 		sub {
 			my ($data, $key, $index, $value, $rest )=@_;
-			die "trie to retrieve an index $index from a no array value (in key $key)\n";
+			croak "trie to retrieve an index $index from a no array value (in key $key)\n";
 		}
 
 	, retrieve_key_from_non_hash    =>
 		sub {
 			my ($data, $key, $index, $value, $rest )=@_;
-			die "trie to retrieve a key from a no hash value (in key $key)\n";
+			croak "trie to retrieve a key from a no hash value (in key $key)\n";
+		}
+	, not_a_coderef_or_method    => $callback->{not_a_coderef_or_method} ||
+		sub {
+			my ($data, $key, $index, $value, $rest )=@_;
+			croak "tried to retrieve from a non-existant coderef or method";
 		}
 	}
 
@@ -178,15 +206,26 @@ None by default.
 
 =head1 SEE ALSO
 
-	XPath 
+	XPath
+ 
+=head1 TODO
+
+Slices of data through /foo[*]/bar syntax. eg. retrieve all the bar keys from each element of the foo
+array.
+
+allow accessing the results of coderefs eg. /foo()/bar retreive the result of the foo coderef/method
+of the object
 
 =head1 AUTHOR
 
 Marco Schrieck, E<lt>marco.schrieck@gmx.deE<gt>
 
+Jeremy Wall L<< jeremy@marzhillstudios.com >>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2006 by Marco Schrieck
+Copyright (C) 2007 by Jeremy Wall
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.4 or,
